@@ -1155,8 +1155,8 @@ public:
         See also the helper function getXmlFromBinary() for loading settings as XML.
 
         In the case that this AudioProcessor is implementing a VST3 that has declared compatible
-        plugins via VST3ClientExtensions::getCompatibleClasses(), the state passed to this
-        function may have been created by one of these compatible plugins.
+        plugins via JUCE_VST3_COMPATIBLE_CLASSES, the state passed to this function may have
+        been created by one of these compatible plugins.
 
         If the parameter IDs of the current plugin differ from the IDs of the plugin whose state
         was passed to this function, you can use information from the plugin state
@@ -1483,11 +1483,8 @@ protected:
     /** @internal */
     std::atomic<AudioPlayHead*> playHead { nullptr };
 
-    /** @internal */
-    void sendParamChangeMessageToListeners (int parameterIndex, float newValue);
-
 public:
-   #ifndef DOXYGEN
+    /** @cond */
     // These methods are all deprecated in favour of using AudioProcessorParameter
     // and AudioProcessorParameterGroup
     [[deprecated]] virtual int getNumParameters();
@@ -1506,9 +1503,6 @@ public:
     [[deprecated]] virtual bool isParameterAutomatable (int parameterIndex) const;
     [[deprecated]] virtual bool isMetaParameter (int parameterIndex) const;
     [[deprecated]] virtual AudioProcessorParameter::Category getParameterCategory (int parameterIndex) const;
-    [[deprecated]] void beginParameterChangeGesture (int parameterIndex);
-    [[deprecated]] void endParameterChangeGesture (int parameterIndex);
-    [[deprecated]] void setParameterNotifyingHost (int parameterIndex, float newValue);
 
     // These functions are deprecated: your audio processor can inform the host
     // on its bus and channel layouts and names using the AudioChannelSet and various bus classes.
@@ -1520,7 +1514,32 @@ public:
     [[deprecated]] virtual const String getOutputChannelName (int channelIndex) const;
     [[deprecated]] virtual bool isInputChannelStereoPair  (int index) const;
     [[deprecated]] virtual bool isOutputChannelStereoPair (int index) const;
-   #endif
+    /** @endcond */
+
+    /** @internal
+        Used to convert new-style per-parameter callbacks into old-style processor-change
+        callbacks.
+        This type will be removed in the future!
+    */
+    class ParameterChangeForwarder : public AudioProcessorParameter::Listener
+    {
+    public:
+        explicit ParameterChangeForwarder (AudioProcessor* o) : owner (o) {}
+
+        ParameterChangeForwarder (const ParameterChangeForwarder& other) : owner (other.owner) {}
+
+        ParameterChangeForwarder& operator= (const ParameterChangeForwarder& other)
+        {
+            owner = other.owner;
+            return *this;
+        }
+
+        void parameterValueChanged (int, float) override;
+        void parameterGestureChanged (int, bool) override;
+
+    private:
+        AudioProcessor* owner = nullptr;
+    };
 
 private:
     //==============================================================================
@@ -1594,6 +1613,8 @@ private:
     AudioProcessorParameterGroup parameterTree;
     Array<AudioProcessorParameter*> flatParameterList;
 
+    ParameterChangeForwarder parameterListener { this };
+
     AudioProcessorParameter* getParamChecked (int) const;
 
   #if JUCE_DEBUG
@@ -1621,7 +1642,6 @@ private:
     template <typename floatType>
     void processBypassed (AudioBuffer<floatType>&, MidiBuffer&);
 
-    friend class AudioProcessorParameter;
     friend class LADSPAPluginInstance;
 
     [[deprecated ("This method is no longer used - you can delete it from your AudioProcessor classes.")]]
