@@ -854,11 +854,11 @@ public:
     using ScopedLockType = typename TypeOfCriticalSectionToUse::ScopedLockType;
 
     //==============================================================================
-   #ifndef DOXYGEN
+    /** @cond */
     [[deprecated ("This method has been replaced by a more flexible templated version and renamed "
                  "to swapWith to be more consistent with the names used in other classes.")]]
     void swapWithArray (ReferenceCountedArray& other) noexcept { swapWith (other); }
-   #endif
+    /** @endcond */
 
 private:
     //==============================================================================
