@@ -259,15 +259,28 @@ public:
         return StringArray (fallbacks.data(), (int) fallbacks.size());
     }
 
-    String getTypefaceName() const             { return options.getName(); }
-    String getTypefaceStyle() const            { return options.getStyle(); }
-    float getHeight() const                    { return options.getHeight(); }
-    float getPointHeight() const               { return options.getPointHeight(); }
-    float getHorizontalScale() const           { return options.getHorizontalScale(); }
-    float getKerning() const                   { return options.getKerningFactor(); }
-    bool getUnderline() const                  { return options.getUnderline(); }
-    bool getFallbackEnabled() const            { return options.getFallbackEnabled(); }
-    TypefaceMetricsKind getMetricsKind() const { return options.getMetricsKind(); }
+    String getTypefaceName() const               { return options.getName(); }
+    String getTypefaceStyle() const              { return options.getStyle(); }
+    float getHeight() const                      { return options.getHeight(); }
+    float getPointHeight() const                 { return options.getPointHeight(); }
+    float getHorizontalScale() const             { return options.getHorizontalScale(); }
+    float getKerning() const                     { return options.getKerningFactor(); }
+    bool getUnderline() const                    { return options.getUnderline(); }
+    bool getFallbackEnabled() const              { return options.getFallbackEnabled(); }
+    TypefaceMetricsKind getMetricsKind() const   { return options.getMetricsKind(); }
+    auto getFeatureSettings() const              { return options.getFeatureSettings(); }
+
+    void setFeatureSetting (const FontFeatureSetting& feature)
+    {
+        jassert (getReferenceCount() == 1);
+        options = options.withFeatureSetting (feature);
+    }
+
+    void removeFeatureSetting (FontFeatureTag feature)
+    {
+        jassert (getReferenceCount() == 1);
+        options = options.withFeatureRemoved (feature);
+    }
 
     std::optional<float> getAscentOverride() const  { return options.getAscentOverride(); }
     std::optional<float> getDescentOverride() const { return options.getDescentOverride(); }
@@ -728,6 +741,7 @@ std::optional<float> Font::getAscentOverride() const noexcept
 
 void Font::setAscentOverride (std::optional<float> x)
 {
+    dupeInternalIfShared();
     font->setAscentOverride (x);
 }
 
@@ -738,6 +752,7 @@ std::optional<float> Font::getDescentOverride() const noexcept
 
 void Font::setDescentOverride (std::optional<float> x)
 {
+    dupeInternalIfShared();
     font->setDescentOverride (x);
 }
 
@@ -750,6 +765,23 @@ bool Font::isUnderlined() const noexcept    { return font->getUnderline(); }
 
 TypefaceMetricsKind Font::getMetricsKind() const noexcept { return font->getMetricsKind(); }
 
+Span<const FontFeatureSetting> Font::getFeatureSettings() const&
+{
+    return font->getFeatureSettings();
+}
+
+void Font::setFeatureSetting (FontFeatureSetting featureSetting)
+{
+    dupeInternalIfShared();
+    font->setFeatureSetting (featureSetting);
+}
+
+void Font::removeFeatureSetting (FontFeatureTag featureToRemove)
+{
+    dupeInternalIfShared();
+    font->removeFeatureSetting (featureToRemove);
+}
+
 void Font::setBold (const bool shouldBeBold)
 {
     auto flags = getStyleFlags();
