@@ -256,7 +256,7 @@ private:
             auto onFocus = [&item]
             {
                 item.parentWindow.disableMouseMovesOnMenuAndAncestors();
-                item.parentWindow.ensureItemComponentIsVisible (item, -1);
+                item.parentWindow.ensureItemComponentIsVisible (item, std::nullopt);
                 item.parentWindow.setCurrentlyHighlightedChild (&item);
             };
 
@@ -424,23 +424,22 @@ struct MenuWindow final : public Component
 
         if (auto visibleID = options.getItemThatMustBeVisible())
         {
-            for (auto* item : items)
+            const auto iter = std::find_if (items.begin(), items.end(), [&] (auto* item)
             {
-                if (item->item.itemID == visibleID)
+                return item->item.itemID == visibleID;
+            });
+
+            if (iter != items.end())
+            {
+                const auto targetPosition = std::invoke ([&]
                 {
-                    const auto targetPosition = [&]
-                    {
-                        if (auto* pc = options.getParentComponent())
-                            return pc->getLocalPoint (nullptr, targetArea.getTopLeft());
+                    if (auto* pc = options.getParentComponent())
+                        return pc->getLocalPoint (nullptr, targetArea.getTopLeft());
 
-                        return targetArea.getTopLeft();
-                    }();
+                    return targetArea.getTopLeft();
+                });
 
-                    auto y = targetPosition.getY() - windowPos.getY();
-                    ensureItemComponentIsVisible (*item, isPositiveAndBelow (y, windowPos.getHeight()) ? y : -1);
-
-                    break;
-                }
+                ensureItemComponentIsVisible (**iter, targetPosition.getY() - windowPos.getY());
             }
         }
 
@@ -1091,40 +1090,68 @@ struct MenuWindow final : public Component
         return correctColumnWidths (maxMenuW);
     }
 
-    void ensureItemComponentIsVisible (const ItemComponent& itemComp, int wantedY)
+    void ensureItemComponentIsVisible (const ItemComponent& itemComp, std::optional<int> wantedY)
     {
-        if (windowPos.getHeight() > PopupMenuSettings::scrollZone * 4)
+        const auto parentArea = getParentArea (windowPos.getPosition(), options.getParentComponent()) / scaleFactor;
+
+        if (const auto posAndOffset = computePosAndOffsetToEnsureVisibility (windowPos, parentArea, itemComp.getBounds(), contentHeight, wantedY))
         {
-            auto currentY = itemComp.getY();
-
-            if (wantedY > 0 || currentY < 0 || itemComp.getBottom() > windowPos.getHeight())
-            {
-                if (wantedY < 0)
-                    wantedY = jlimit (PopupMenuSettings::scrollZone,
-                                      jmax (PopupMenuSettings::scrollZone,
-                                            windowPos.getHeight() - (PopupMenuSettings::scrollZone + itemComp.getHeight())),
-                                      currentY);
-
-                auto parentArea = getParentArea (windowPos.getPosition(), options.getParentComponent()) / scaleFactor;
-                auto deltaY = wantedY - currentY;
-
-                windowPos.setSize (jmin (windowPos.getWidth(), parentArea.getWidth()),
-                                   jmin (windowPos.getHeight(), parentArea.getHeight()));
-
-                auto newY = jlimit (parentArea.getY(),
-                                    parentArea.getBottom() - windowPos.getHeight(),
-                                    windowPos.getY() + deltaY);
-
-                deltaY -= newY - windowPos.getY();
-
-                childYOffset -= deltaY;
-                windowPos.setPosition (windowPos.getX(), newY);
-
-                updateYPositions();
-            }
+            std::tie (windowPos, childYOffset) = std::tie (posAndOffset->windowPos, posAndOffset->childYOffset);
+            updateYPositions();
         }
     }
 
+    struct PosAndOffset
+    {
+        Rectangle<int> windowPos;
+        int childYOffset = 0;
+    };
+
+    static std::optional<PosAndOffset> computePosAndOffsetToEnsureVisibility (Rectangle<int> windowPos,
+                                                                              const Rectangle<int>& parentArea,
+                                                                              const Rectangle<int>& itemCompBounds,
+                                                                              int contentHeight,
+                                                                              std::optional<int> wantedY)
+    {
+        // If there's no specific wantedY, and the item component is already visible, then we don't
+        // need to make any adjustments.
+        if (! wantedY.has_value() && 0 <= itemCompBounds.getY() && itemCompBounds.getBottom() <= windowPos.getHeight())
+            return {};
+
+        const auto spaceNeededAboveItem = jmin (PopupMenuSettings::scrollZone, itemCompBounds.getY());
+        const auto spaceNeededBelowItem = jmin (PopupMenuSettings::scrollZone, contentHeight - itemCompBounds.getBottom());
+        const auto parentSpaceTargetY = windowPos.getY() + wantedY.value_or (itemCompBounds.getY());
+
+        // In order to display the visible item over the target area, we need to make sure that
+        // there's enough space above and below to hold the scroll areas if they're showing.
+        // Ideally, we want to avoid the case where the menu opens with the scroll area over the
+        // target area.
+        const auto isSpaceToOverlay = spaceNeededAboveItem <= (parentSpaceTargetY - parentArea.getY())
+                                   && spaceNeededBelowItem <= (parentArea.getBottom() - (parentSpaceTargetY + itemCompBounds.getHeight()));
+
+        if (wantedY.has_value() && isSpaceToOverlay)
+        {
+            windowPos = windowPos.withY (parentSpaceTargetY - itemCompBounds.getY())
+                                 .withHeight (contentHeight)
+                                 .constrainedWithin (parentArea);
+
+            const auto menuSpaceTargetY = parentSpaceTargetY - windowPos.getY();
+            const auto offset = itemCompBounds.getY() - menuSpaceTargetY;
+
+            return PosAndOffset { windowPos, offset };
+        }
+
+        // If there's not enough space to overlay the menu, then just use the provided menu
+        // bounds but try to position the visible item as close to the target area as possible,
+        // while avoiding the scroll areas.
+        const auto menuSpaceTargetY = jlimit (spaceNeededAboveItem,
+                                              windowPos.getHeight() - spaceNeededBelowItem - itemCompBounds.getHeight(),
+                                              parentSpaceTargetY - windowPos.getY());
+        const auto offset = itemCompBounds.getY() - menuSpaceTargetY;
+
+        return PosAndOffset { windowPos, offset };
+    }
+
     void resizeToBestWindowPos()
     {
         auto r = windowPos;
